@@ -127,13 +127,6 @@ QString SyncJournalDb::makeDbName(const QString &localPath,
     return journalPath;
 }
 
-QString SyncJournalDb::makeDbName(const QString &localPath, const QUrl &remoteUrl, const QString &remotePath, const QString &user)
-{
-    // legacy name only used by the cmd client
-    const QString key = QStringLiteral("%1@%2:%3").arg(user, remoteUrl.toString(), remotePath);
-    return makeDbName(localPath, QString::fromUtf8(QCryptographicHash::hash(key.toUtf8(), QCryptographicHash::Md5).left(6).toHex()));
-}
-
 bool SyncJournalDb::maybeMigrateDb(const QString &localPath, const QString &absoluteJournalPath)
 {
     const QString oldDbName = localPath + QLatin1String(".csync_journal.db");
@@ -1614,18 +1607,39 @@ int SyncJournalDb::wipeErrorBlacklist()
     return -1;
 }
 
-void SyncJournalDb::wipeErrorBlacklistEntry(const QString &file)
+void SyncJournalDb::wipeErrorBlacklistEntry(const QString &relativeFile)
 {
-    if (file.isEmpty()) {
+    if (relativeFile.isEmpty()) {
         return;
     }
+    Q_ASSERT(QFileInfo(relativeFile).isRelative());
 
     QMutexLocker locker(&_mutex);
     if (checkConnect()) {
         SqlQuery query(_db);
 
         query.prepare("DELETE FROM blacklist WHERE path=?1");
-        query.bindValue(1, file);
+        query.bindValue(1, relativeFile);
+        if (!query.exec()) {
+            sqlFail(QStringLiteral("Deletion of blacklist item failed."), query);
+        }
+    }
+}
+
+void SyncJournalDb::wipeErrorBlacklistEntry(const QString &relativeFile, SyncJournalErrorBlacklistRecord::Category category)
+{
+    if (relativeFile.isEmpty()) {
+        return;
+    }
+    Q_ASSERT(QFileInfo(relativeFile).isRelative());
+
+    QMutexLocker locker(&_mutex);
+    if (checkConnect()) {
+        SqlQuery query(_db);
+
+        query.prepare("DELETE FROM blacklist WHERE path=?1 AND errorCategory=?2");
+        query.bindValue(1, relativeFile);
+        query.bindValue(2, category);
         if (!query.exec()) {
             sqlFail(QStringLiteral("Deletion of blacklist item failed."), query);
         }
