@@ -36,6 +36,8 @@
 #include <QSet>
 #include <QNetworkProxy>
 
+using namespace std::chrono_literals;
+
 namespace {
 /*
  * [Accounts]
@@ -146,9 +148,11 @@ FolderMan::FolderMan(QObject *parent)
     _socketApi.reset(new SocketApi);
 
     // Set the remote poll interval fixed to 10 seconds.
-    // That does not mean that it polls every 10 seconds, but it checks every 10 secs
-    // if one of the folders is due to sync.
-    _etagPollTimer.setInterval(1000);
+    // That does not mean that it polls every 10 seconds, but it checks every 10 seconds
+    // if one of the folders is due to sync. This means that if the server advertises a
+    // pollinterval that is not a multiple of 10 seconds, then that pollinterval will be
+    // rounded up to the next 10 seconds in practice. 10-second granularity is acceptable.
+    _etagPollTimer.setInterval(10s);
     QObject::connect(&_etagPollTimer, &QTimer::timeout, this, &FolderMan::slotEtagPollTimerTimeout);
     _etagPollTimer.start();
 
@@ -189,24 +193,26 @@ void FolderMan::unloadFolder(Folder *f)
 {
     Q_ASSERT(f);
 
+    _folderMap.remove(f->alias());
     _socketApi->slotUnregisterPath(f->alias());
 
-    _folderMap.remove(f->alias());
 
-    disconnect(f, &Folder::syncStarted,
-        this, &FolderMan::slotFolderSyncStarted);
-    disconnect(f, &Folder::syncFinished,
-        this, &FolderMan::slotFolderSyncFinished);
-    disconnect(f, &Folder::syncStateChange,
-        this, &FolderMan::slotForwardFolderSyncStateChange);
-    disconnect(f, &Folder::syncPausedChanged,
-        this, &FolderMan::slotFolderSyncPaused);
-    disconnect(&f->syncEngine().syncFileStatusTracker(), &SyncFileStatusTracker::fileStatusChanged,
-        _socketApi.data(), &SocketApi::broadcastStatusPushMessage);
-    disconnect(f, &Folder::watchedFileChangedExternally,
-        &f->syncEngine().syncFileStatusTracker(), &SyncFileStatusTracker::slotPathTouched);
+    if (!f->hasSetupError()) {
+        disconnect(f, &Folder::syncStarted,
+            this, &FolderMan::slotFolderSyncStarted);
+        disconnect(f, &Folder::syncFinished,
+            this, &FolderMan::slotFolderSyncFinished);
+        disconnect(f, &Folder::syncStateChange,
+            this, &FolderMan::slotForwardFolderSyncStateChange);
+        disconnect(f, &Folder::syncPausedChanged,
+            this, &FolderMan::slotFolderSyncPaused);
+        disconnect(&f->syncEngine().syncFileStatusTracker(), &SyncFileStatusTracker::fileStatusChanged,
+            _socketApi.data(), &SocketApi::broadcastStatusPushMessage);
+        disconnect(f, &Folder::watchedFileChangedExternally,
+            &f->syncEngine().syncFileStatusTracker(), &SyncFileStatusTracker::slotPathTouched);
 
-    f->syncEngine().disconnect(f);
+        f->syncEngine().disconnect(f);
+    }
 }
 
 void FolderMan::unloadAndDeleteAllFolders()
@@ -909,13 +915,13 @@ void FolderMan::slotEtagPollTimerTimeout()
     }
 }
 
-void FolderMan::slotRemoveFoldersForAccount(AccountState *accountState)
+void FolderMan::slotRemoveFoldersForAccount(const AccountStatePtr &accountState)
 {
     QList<Folder *> foldersToRemove;
     // reserve a magic number
     foldersToRemove.reserve(16);
     for (auto *folder : qAsConst(_folderMap)) {
-        if (folder->accountState() == accountState) {
+        if (folder->accountState() == accountState.data()) {
             foldersToRemove.append(folder);
         }
     }
@@ -1119,18 +1125,20 @@ Folder *FolderMan::addFolderInternal(
     }
 
     // See matching disconnects in unloadFolder().
-    connect(folder, &Folder::syncStarted, this, &FolderMan::slotFolderSyncStarted);
-    connect(folder, &Folder::syncFinished, this, &FolderMan::slotFolderSyncFinished);
-    connect(folder, &Folder::syncStateChange, this, &FolderMan::slotForwardFolderSyncStateChange);
-    connect(folder, &Folder::syncPausedChanged, this, &FolderMan::slotFolderSyncPaused);
-    connect(folder, &Folder::canSyncChanged, this, &FolderMan::slotFolderCanSyncChanged);
-    connect(&folder->syncEngine().syncFileStatusTracker(), &SyncFileStatusTracker::fileStatusChanged,
-        _socketApi.data(), &SocketApi::broadcastStatusPushMessage);
-    connect(folder, &Folder::watchedFileChangedExternally,
-        &folder->syncEngine().syncFileStatusTracker(), &SyncFileStatusTracker::slotPathTouched);
+    if (!folder->hasSetupError()) {
+        connect(folder, &Folder::syncStarted, this, &FolderMan::slotFolderSyncStarted);
+        connect(folder, &Folder::syncFinished, this, &FolderMan::slotFolderSyncFinished);
+        connect(folder, &Folder::syncStateChange, this, &FolderMan::slotForwardFolderSyncStateChange);
+        connect(folder, &Folder::syncPausedChanged, this, &FolderMan::slotFolderSyncPaused);
+        connect(folder, &Folder::canSyncChanged, this, &FolderMan::slotFolderCanSyncChanged);
+        connect(&folder->syncEngine().syncFileStatusTracker(), &SyncFileStatusTracker::fileStatusChanged,
+            _socketApi.data(), &SocketApi::broadcastStatusPushMessage);
+        connect(folder, &Folder::watchedFileChangedExternally,
+            &folder->syncEngine().syncFileStatusTracker(), &SyncFileStatusTracker::slotPathTouched);
 
-    folder->registerFolderWatcher();
-    registerFolderWithSocketApi(folder);
+        folder->registerFolderWatcher();
+        registerFolderWithSocketApi(folder);
+    }
     return folder;
 }
 
