@@ -40,6 +40,8 @@
 
 #include "creds/abstractcredentials.h"
 #include "creds/httpcredentials.h"
+#include "theme.h"
+
 
 namespace OCC {
 
@@ -415,7 +417,6 @@ const QHash<QString, qint64> &LsColJob::sizes() const
 
 CheckServerJob::CheckServerJob(AccountPtr account, QObject *parent)
     : AbstractNetworkJob(account, QStringLiteral("status.php"), parent)
-    , _subdirFallback(false)
 {
     setIgnoreCredentialFailure(true);
     setAuthenticationJob(true);
@@ -430,18 +431,19 @@ void CheckServerJob::start()
     req.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
     req.setRawHeader(QByteArrayLiteral("OC-Connection-Validator"), QByteArrayLiteral("desktop"));
     req.setMaximumRedirectsAllowed(_maxRedirectsAllowed);
+
+    if (_clearCookies && Theme::instance()->connectionValidatorClearCookies()) {
+        _account->clearCookieJar();
+    }
     sendRequest("GET", Utility::concatUrlPath(_serverUrl, path()), req);
-    connect(reply(), &QNetworkReply::metaDataChanged, this, &CheckServerJob::metaDataChangedSlot);
-    connect(reply(), &QNetworkReply::encrypted, this, &CheckServerJob::encryptedSlot);
-    connect(reply(), &QNetworkReply::redirected, this, [this] {
-        const auto code = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-        if (code == 302 || code == 307) {
-            _redirectDistinct = false;
-        }
-    });
     AbstractNetworkJob::start();
 }
 
+void CheckServerJob::setClearCookies(bool clearCookies)
+{
+    _clearCookies = clearCookies;
+}
+
 void CheckServerJob::onTimedOut()
 {
     qCWarning(lcCheckServerJob) << "TIMEOUT";
@@ -486,6 +488,18 @@ void CheckServerJob::encryptedSlot()
     mergeSslConfigurationForSslButton(reply()->sslConfiguration(), account());
 }
 
+void CheckServerJob::newReplyHook(QNetworkReply *reply)
+{
+    connect(reply, &QNetworkReply::metaDataChanged, this, &CheckServerJob::metaDataChangedSlot);
+    connect(reply, &QNetworkReply::encrypted, this, &CheckServerJob::encryptedSlot);
+    connect(reply, &QNetworkReply::redirected, this, [reply, this] {
+        const auto code = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+        if (code == 302 || code == 307) {
+            _redirectDistinct = false;
+        }
+    });
+}
+
 int CheckServerJob::maxRedirectsAllowed() const
 {
     return _maxRedirectsAllowed;
@@ -843,6 +857,11 @@ void SimpleNetworkJob::start()
     AbstractNetworkJob::start();
 }
 
+void SimpleNetworkJob::addNewReplyHook(std::function<void(QNetworkReply *)> &&hook)
+{
+    _replyHooks.push_back(hook);
+}
+
 void SimpleNetworkJob::prepareRequest(const QByteArray &verb, const QUrl &url,
     const QNetworkRequest &req, QIODevice *requestBody)
 {
@@ -878,6 +897,13 @@ bool SimpleNetworkJob::finished()
     return true;
 }
 
+void SimpleNetworkJob::newReplyHook(QNetworkReply *reply)
+{
+    for (const auto &hook : _replyHooks) {
+        hook(reply);
+    }
+}
+
 void fetchPrivateLinkUrl(AccountPtr account, const QString &remotePath, QObject *target,
     std::function<void(const QString &url)> targetFun)
 {
