@@ -2,7 +2,7 @@
 import names
 import os
 import sys
-from os import listdir
+from os import listdir, rename
 from os.path import isfile, join, isdir
 import re
 import urllib.request
@@ -36,6 +36,15 @@ socketConnect = None
 
 stateDataFromMiddleware = None
 
+# File syncing in client has the following status
+SYNC_STATUS = {
+    'SYNC': 'STATUS:SYNC',  # sync in process
+    'OK': 'STATUS:OK',  # sync completed
+    'ERROR': 'STATUS:ERROR',  # file sync has error
+    'IGNORE': 'STATUS:IGNORE',  # file is igored
+    'NOP': 'STATUS:NOP',  # file yet to be synced
+}
+
 
 def getTestStateFromMiddleware(context):
     global stateDataFromMiddleware
@@ -134,7 +143,8 @@ def step(context):
     newAccount.selectSyncFolder(context)
 
 
-def isItemSynced(type, itemName):
+# Using socket API to check file sync status
+def hasSyncStatus(type, itemName, status):
     if type != 'FILE' and type != 'FOLDER':
         raise Exception("type must be 'FILE' or 'FOLDER'")
     socketConnect = syncstate.SocketConnect()
@@ -143,35 +153,114 @@ def isItemSynced(type, itemName):
     if not socketConnect.read_socket_data_with_timeout(0.1):
         return False
     for line in socketConnect.get_available_responses():
-        if line.startswith('STATUS:OK') and line.endswith(itemName):
+        if line.startswith(status) and line.endswith(itemName):
             return True
         elif line.endswith(itemName):
             return False
 
 
-def isFolderSynced(folderName):
-    return isItemSynced('FOLDER', folderName)
+def folderHasSyncStatus(folderName, status):
+    return hasSyncStatus('FOLDER', folderName, status)
 
 
-def isFileSynced(fileName):
-    return isItemSynced('FILE', fileName)
+def fileHasSyncStatus(fileName, status):
+    return hasSyncStatus('FILE', fileName, status)
 
 
-def waitForFileToBeSynced(context, fileName):
-    waitFor(
-        lambda: isFileSynced(
-            sanitizePath(context.userData['currentUserSyncPath'] + fileName)
-        ),
-        context.userData['clientSyncTimeout'] * 1000,
+def waitForFileOrFolderToHaveSyncStatus(
+    context, resource, resourceType, status=SYNC_STATUS['OK'], timeout=None
+):
+    if not timeout:
+        timeout = context.userData['maxSyncTimeout'] * 1000
+
+    resource = join(context.userData['currentUserSyncPath'], resource)
+    if resourceType.lower() == "file":
+        result = waitFor(
+            lambda: fileHasSyncStatus(sanitizePath(resource), status),
+            timeout,
+        )
+    elif resourceType.lower() == "folder":
+        result = waitFor(
+            lambda: folderHasSyncStatus(sanitizePath(resource), status),
+            timeout,
+        )
+
+    if not result:
+        if status == SYNC_STATUS['ERROR']:
+            expected = "have sync error"
+        elif status == SYNC_STATUS['IGNORE']:
+            expected = "be sync ignored"
+        else:
+            expected = "be synced"
+        raise Exception(
+            "Expected "
+            + resourceType
+            + " '"
+            + resource
+            + "' to "
+            + expected
+            + ", but not."
+        )
+
+
+def waitForSyncToStart(context, resource, resourceType):
+    resource = join(context.userData['currentUserSyncPath'], resource)
+
+    hasStatusNOP = hasSyncStatus(resourceType.upper(), resource, SYNC_STATUS['NOP'])
+    hasStatusSYNC = hasSyncStatus(resourceType.upper(), resource, SYNC_STATUS['SYNC'])
+
+    if hasStatusSYNC:
+        return
+
+    try:
+        if hasStatusNOP:
+            waitForFileOrFolderToHaveSyncStatus(
+                context, resource, resourceType, SYNC_STATUS['SYNC']
+            )
+        else:
+            waitForFileOrFolderToHaveSyncStatus(
+                context,
+                resource,
+                resourceType,
+                SYNC_STATUS['SYNC'],
+                context.userData['minSyncTimeout'] * 1000,
+            )
+    except:
+        hasStatusNOP = hasSyncStatus(resourceType.upper(), resource, SYNC_STATUS['NOP'])
+        if hasStatusNOP:
+            raise Exception(
+                "Expected "
+                + resourceType
+                + " '"
+                + resource
+                + "' to have sync started but not."
+            )
+
+
+def waitForFileOrFolderToSync(context, resource, resourceType):
+    waitForSyncToStart(context, resource, resourceType)
+    waitForFileOrFolderToHaveSyncStatus(
+        context, resource, resourceType, SYNC_STATUS['OK']
     )
 
 
-def waitForFolderToBeSynced(context, folderName):
-    waitFor(
-        lambda: isFolderSynced(
-            sanitizePath(context.userData['currentUserSyncPath'] + folderName)
-        ),
-        context.userData['clientSyncTimeout'] * 1000,
+def waitForRootFolderToSync(context):
+    waitForFileOrFolderToSync(
+        context, context.userData['currentUserSyncPath'], 'folder'
+    )
+
+
+def waitForFileOrFolderToHaveSyncError(context, resource, resourceType):
+    waitForSyncToStart(context, resource, resourceType)
+    waitForFileOrFolderToHaveSyncStatus(
+        context, resource, resourceType, SYNC_STATUS['ERROR']
+    )
+
+
+def waitForFileOrFolderToBeSyncIgnored(context, resource, resourceType):
+    waitForSyncToStart(context, resource, resourceType)
+    waitForFileOrFolderToHaveSyncStatus(
+        context, resource, resourceType, SYNC_STATUS['IGNORE']
     )
 
 
@@ -242,10 +331,10 @@ def step(context, resource):
     shareItem = SharingDialog()
 
     # In the following loop we are trying to share resource with given permission to one user at a time given from the data table in the feature file
-    for row in context.table[1:]:
+    for count, row in enumerate(context.table[1:]):
         receiver = row[0]
         permissions = row[1]
-        shareItem.addCollaborator(receiver, permissions)
+        shareItem.addCollaborator(receiver, permissions, False, count + 1)
 
     shareItem.closeSharingDialog()
 
@@ -331,22 +420,27 @@ def collaboratorShouldBeListed(
 
 @When('the user waits for the files to sync')
 def step(context):
-    waitForFolderToBeSynced(context, '/')
+    waitForRootFolderToSync(context)
 
 
-@When('the user waits for file "|any|" to be synced')
-def step(context, fileName):
-    waitForFileToBeSynced(context, fileName)
+@When(r'the user waits for (file|folder) "([^"]*)" to be synced', regexp=True)
+def step(context, type, resource):
+    waitForFileOrFolderToSync(context, resource, type)
 
 
-@When('the user waits for folder "|any|" to be synced')
-def step(context, folderName):
-    waitForFolderToBeSynced(context, folderName)
+@When(r'the user waits for (file|folder) "([^"]*)" to have sync error', regexp=True)
+def step(context, type, resource):
+    waitForFileOrFolderToHaveSyncError(context, resource, type)
 
 
-@Given('the user has waited for file "|any|" to be synced')
-def step(context, fileName):
-    waitForFileToBeSynced(context, fileName)
+@When(r'the user waits for (file|folder) "([^"]*)" to be sync ignored', regexp=True)
+def step(context, type, resource):
+    waitForFileOrFolderToBeSyncIgnored(context, resource, type)
+
+
+@Given(r'the user has waited for (file|folder) "([^"]*)" to be synced', regexp=True)
+def step(context, type, resource):
+    waitForFileOrFolderToSync(context, resource, type)
 
 
 @Given(
@@ -370,6 +464,11 @@ def createFile(context, filename, username=None):
         syncPath = getUserSyncPath(context, username)
     else:
         syncPath = context.userData['currentUserSyncPath']
+
+    # A file is scheduled to be synced but is marked as ignored for 5 seconds. And if we try to sync it, it will fail. So we need to wait for 5 seconds.
+    # https://github.com/owncloud/client/issues/9325
+    snooze(context.userData['minSyncTimeout'])
+
     f = open(join(syncPath, filename), "w")
     f.write(fileContent)
     f.close()
@@ -395,6 +494,12 @@ def createFolder(context, foldername, username=None):
     os.makedirs(path)
 
 
+def renameFileFolder(context, source, destination):
+    source = join(context.userData['currentUserSyncPath'], source)
+    destination = join(context.userData['currentUserSyncPath'], destination)
+    rename(source, destination)
+
+
 @When('the user copies the folder "|any|" to "|any|"')
 def step(context, sourceFolder, destinationFolder):
     source_dir = join(context.userData['currentUserSyncPath'], sourceFolder)
@@ -402,6 +507,11 @@ def step(context, sourceFolder, destinationFolder):
     shutil.copytree(source_dir, destination_dir)
 
 
+@When(r'the user renames a (file|folder) "([^"]*)" to "([^"]*)"', regexp=True)
+def step(context, type, source, destination):
+    renameFileFolder(context, source, destination)
+
+
 @Given(r"^(.*) on the server (.*)$", regexp=True)
 def step(context, stepPart1, stepPart2):
     executeStepThroughMiddleware(context, "Given " + stepPart1 + " " + stepPart2)
@@ -447,11 +557,11 @@ def step(context, resourceType, resource):
     resourceExists = False
     if resourceType == 'file':
         resourceExists = fileExists(
-            resourcePath, context.userData['clientSyncTimeout'] * 1000
+            resourcePath, context.userData['maxSyncTimeout'] * 1000
         )
     elif resourceType == 'folder':
         resourceExists = folderExists(
-            resourcePath, context.userData['clientSyncTimeout'] * 1000
+            resourcePath, context.userData['maxSyncTimeout'] * 1000
         )
     else:
         raise Exception("Unsupported resource type '" + resourceType + "'")
@@ -483,7 +593,6 @@ def step(context, resourceType, resource):
 
 @Given('the user has paused the file sync')
 def step(context):
-    waitForFolderToBeSynced(context, '/')
     syncWizard = SyncWizard()
     syncWizard.performAction("Pause sync")
 
@@ -587,22 +696,7 @@ def step(context, tabName):
 
 def openSharingDialog(context, resource, itemType='file'):
     resource = getResourcePath(context, resource)
-
-    if itemType == 'folder':
-        waitFor(
-            lambda: isFolderSynced(resource),
-            context.userData['clientSyncTimeout'] * 1000,
-        )
-    elif itemType == 'file':
-        waitFor(
-            lambda: isFileSynced(resource), context.userData['clientSyncTimeout'] * 1000
-        )
-    else:
-        raise Exception("No such item type for resource")
-
-    waitFor(
-        lambda: shareResource(resource), context.userData['clientSyncTimeout'] * 1000
-    )
+    waitFor(lambda: shareResource(resource), context.userData['maxSyncTimeout'] * 1000)
 
 
 @When('the user opens the public links dialog of "|any|" using the client-UI')
@@ -685,6 +779,21 @@ def step(context, resource, password):
     createPublicLinkShare(context, resource, password)
 
 
+@Then('the expiration date of the last public link of file "|any|" should be "|any|"')
+def step(context, resource, expiryDate):
+    openSharingDialog(context, resource)
+    publicLinkDialog = PublicLinkDialog()
+    publicLinkDialog.openPublicLinkDialog()
+
+    if expiryDate.strip("%") == "default":
+        expiryDate = PublicLinkDialog.getDefaultExpiryDate()
+
+    publicLinkDialog.verifyExpirationDate(expiryDate)
+
+    shareItem = SharingDialog()
+    shareItem.closeSharingDialog()
+
+
 def setExpirationDateWithVerification(resource, publicLinkName, expireDate):
     publicLinkDialog = PublicLinkDialog()
     publicLinkDialog.verifyResource(resource)
@@ -721,11 +830,18 @@ def step(context):
     linkSettings = {}
     for row in context.table:
         linkSettings[row[0]] = row[1]
+
+    if "path" not in linkSettings:
+        raise Exception("'path' is required but not given.")
+
+    if "expireDate" in linkSettings and linkSettings['expireDate'] == "%default%":
+        linkSettings['expireDate'] = linkSettings['expireDate'].strip("%")
+
     createPublicLinkShare(
         context,
         resource=linkSettings['path'],
-        password=linkSettings['password'],
-        expireDate=linkSettings['expireDate'],
+        password=linkSettings['password'] if "password" in linkSettings else None,
+        expireDate=linkSettings['expireDate'] if "expireDate" in linkSettings else None,
     )
 
 
@@ -799,10 +915,7 @@ def step(context, username):
 
 @Given('user "|any|" has logged out of the client-UI')
 def step(context, username):
-    waitForFolderToBeSynced(context, '/')
-    # TODO: find some way to dynamically to check if files are synced
-    # It might take some time for all files to sync
-    snooze(5)
+    waitForRootFolderToSync(context)
     accountStatus = AccountStatus(context, getDisplaynameForUser(context, username))
     accountStatus.accountAction("Log out")
     isUserSignedOut(context, username)
@@ -821,7 +934,7 @@ def step(context, username):
 def step(context, username):
     # TODO: find some way to dynamically to check if files are synced
     # It might take some time for all files to sync and connect to ther server
-    snooze(5)
+    snooze(context.userData['minSyncTimeout'])
     isUserSignedIn(context, username)
 
 
@@ -831,7 +944,6 @@ def step(context, username, host):
     displayname = substituteInLineCodes(context, displayname)
     host = substituteInLineCodes(context, host)
 
-    waitForFolderToBeSynced(context, '/')
     accountStatus = AccountStatus(context, displayname, host)
     accountStatus.removeConnection()
 
@@ -915,40 +1027,45 @@ def step(context, resource, group):
     sharingDialog.selectCollaborator(group, True)
 
 
+def overwriteFile(resource, content):
+    f = open(resource, "w")
+    f.write(content)
+    f.close()
+
+
+def tryToOverwriteFile(context, resource, content):
+    try:
+        overwriteFile(resource, content)
+    except:
+        pass
+
+
 @When('the user overwrites the file "|any|" with content "|any|"')
 def step(context, resource, content):
     print("starting file overwrite")
-    waitForFileToBeSynced(context, resource)
-    waitForFolderToBeSynced(context, '/')
+    resource = join(context.userData['currentUserSyncPath'], resource)
 
     # overwriting the file immediately after it has been synced from the server seems to have some problem.
     # The client does not see the change although the changes have already been made thus we are having a race condition
     # So for now we add 5sec static wait
     # an issue https://github.com/owncloud/client/issues/8832 has been created for it
+    snooze(context.userData['minSyncTimeout'])
 
-    snooze(5)
-
-    f = open(context.userData['currentUserSyncPath'] + resource, "w")
-    f.write(content)
-    f.close()
+    overwriteFile(resource, content)
 
     print("file has been overwritten")
-    waitForFileToBeSynced(context, resource)
 
 
 @When('the user tries to overwrite the file "|any|" with content "|any|"')
 def step(context, resource, content):
-    waitForFileToBeSynced(context, resource)
-    waitForFolderToBeSynced(context, '/')
+    resource = context.userData['currentUserSyncPath'] + resource
+    tryToOverwriteFile(context, resource, content)
 
-    try:
-        f = open(context.userData['currentUserSyncPath'] + resource, "w")
-        f.write(content)
-        f.close()
-    except:
-        pass
 
-    waitForFileToBeSynced(context, resource)
+@When('user "|any|" tries to overwrite the file "|any|" with content "|any|"')
+def step(context, user, resource, content):
+    resource = getResourcePath(context, resource, user)
+    tryToOverwriteFile(context, resource, content)
 
 
 def enableVFSSupport(vfsBtnText):
@@ -1032,6 +1149,11 @@ def step(context, errorMsg):
 
 @When(r'the user deletes the (file|folder) "([^"]*)"', regexp=True)
 def step(context, itemType, resource):
+    # deleting the file immediately after it has been synced from the server seems to have some problem.
+    # issue: https://github.com/owncloud/client/issues/8832
+    if itemType == "file":
+        snooze(context.userData['minSyncTimeout'])
+
     resourcePath = sanitizePath(context.userData['currentUserSyncPath'] + resource)
     if itemType == 'file':
         os.remove(resourcePath)
@@ -1040,6 +1162,21 @@ def step(context, itemType, resource):
     else:
         raise Exception("No such item type for resource")
 
+    isSyncFolderEmpty = True
+    for item in listdir(context.userData['currentUserSyncPath']):
+        # do not count the hidden files as they are ignored by the client
+        if not item.startswith("."):
+            isSyncFolderEmpty = False
+            break
+
+    # if the sync folder is empty after deleting file,
+    # a dialog will popup asking to confirm "Remove all files"
+    if isSyncFolderEmpty:
+        try:
+            AccountStatus.confirmRemoveAllFiles()
+        except:
+            pass
+
 
 @When(
     'the user unshares the resource "|any|" for collaborator "|any|" using the client-UI'
@@ -1265,3 +1402,20 @@ def step(context):
 @Then('VFS enabled baseline image should not match the default screenshot')
 def step(context):
     test.xvp("VP_VFS_enabled")
+
+
+@Given('user "|any|" has created the following files inside the sync folder:')
+def step(context, username):
+    '''
+    Create files without any content
+    '''
+    syncPath = getUserSyncPath(context, username)
+
+    # A file is scheduled to be synced but is marked as ignored for 5 seconds. And if we try to sync it, it will fail. So we need to wait for 5 seconds.
+    # https://github.com/owncloud/client/issues/9325
+    snooze(context.userData['minSyncTimeout'])
+
+    for row in context.table[1:]:
+        filename = syncPath + row[0]
+        f = open(join(syncPath, filename), "w")
+        f.close()
